/************************************************************************
 *                                                                      *
 * This code is for operating the hard drives                           *
 *                                                                      *
 * Copyright (c) 2006, 2007 E. D. MacPherson                            *
 *                                                                      *
 ************************************************************************/

#include "predef.h" 
#include <stdio.h>
#include <startnet.h>
#include <string.h>
#include <bsp.h>
#include "defines.h"
#include "structs2.h"
#include "HardDrives.h"
#include "main.h"
 
#define POINTER(TypeOfPointer, prefix, NameOfPointer) volatile TypeOfPointer * prefix ## NameOfPointer = (TypeOfPointer *) NameOfPointer;

/***************
 * Hard Drive  *
 ***************/
#define HARDDRIVE_A      (CS2)
#define HARDDRIVE_B      (HARDDRIVE_A + 0x10)
#define HD_DATA          (HARDDRIVE_A)
#define HD_ERROR         (HARDDRIVE_A + 2)
#define HD_FEATURES      (HARDDRIVE_A + 2)
#define HD_SECTORCNT     (HARDDRIVE_A + 4)
#define HD_LBALOW        (HARDDRIVE_A + 6)
#define HD_LBAMID        (HARDDRIVE_A + 8)
#define HD_LBAHIGH       (HARDDRIVE_A + 0xa)
#define HD_DEVREG        (HARDDRIVE_A + 0xc)
#define HD_STATUS        (HARDDRIVE_A + 0xe)
#define HD_COMMAND       (HARDDRIVE_A + 0xe)
#define HD_ALTSTATUS     (HARDDRIVE_B + 0xc)
#define HD_DEVCONTROL    (HARDDRIVE_B + 0xc)
#define HDRDY            0x40
#define HDBUSY           0x80
#define HDERROR          0x01

POINTER(unsigned short, sp, HD_DATA)           // data
POINTER(unsigned short, sp, HD_ERROR)          // error
POINTER(unsigned short, sp, HD_FEATURES)       // features
POINTER(unsigned short, sp, HD_SECTORCNT)      // sector count
POINTER(unsigned short, sp, HD_LBALOW)         // LBA low byte
POINTER(unsigned short, sp, HD_LBAMID)         // LBA mid byte
POINTER(unsigned short, sp, HD_LBAHIGH)        // LBA high byte
POINTER(unsigned short, sp, HD_DEVREG)         // device register
POINTER(unsigned short, sp, HD_STATUS)         // status
POINTER(unsigned short, sp, HD_COMMAND)        // command
POINTER(unsigned short, sp, HD_ALTSTATUS)      // mirror of status
POINTER(unsigned short, sp, HD_DEVCONTROL)     // device control


extern struct cust       customer;
extern struct game       Game;
extern int               websocket;
extern bool              WebCommands;
extern int               fdlancon;

unsigned char            HDbuf[512];
unsigned short           status;
unsigned char            SectorsPerCluster;
OS_SEM                   HDSem;
char                     filemarker[] = "Ewan MacPherson Server";
char                     buf[128];

void _ReadSectorHD (unsigned int lba, unsigned char * buffer);
void _WriteSectorHD (unsigned int lba, unsigned char * buffer);
void ReadAndDisplay (unsigned int lba);
void WaitForHD (void);
void WaitForHDnBUSY (void);



void HDInit(void)
{
  writestring( fdlancon, "LAN -- Waiting for Hard Drive\n\r");

  OSSemPost( &HDSem );              // set the semaphore to 1

  *spHD_DEVCONTROL = 6;             // set reset and no interrupt
  OSTimeDly(1);                     // wait 50 ms
  *spHD_DEVCONTROL = 2;             // clear reset and still no interrupt

  WaitForHD();

  printf("LAN -- Hard Drive is ready; status = %0x, mirror = %0x\r\n",
         *spHD_STATUS & 0xff, *spHD_ALTSTATUS & 0xff);

  *spHD_LBALOW  = 0xa0;           // test only
  *spHD_LBAMID  = 0xb1;           // test only
  *spHD_LBAHIGH = 0xc2;           // test only
  *spHD_DEVREG  = 0xe0;           // make sure we don't screw up this register

  printf("LAN -- LBA low = %02x, mid = %02x, high = %02x, top = %02x\r\n",
         *spHD_LBALOW & 0xff, *spHD_LBAMID & 0xff, *spHD_LBAHIGH & 0xff, *spHD_DEVREG & 0xff);
 
  WaitForHD();
/*  
  *spHD_COMMAND = 0xec;           // get device characteristics

  printf("\r\n");
  for (int x = 0; x < 256; x++) 
  {
    WaitForHD();

    printf("%03d=%04x,", x, *spHD_DATA);
    if ((x & 7) == 7)
      printf("\r\n");
  }
*/
  OSSemPost( &HDSem );            // set the semaphore to 1
  WaitForHD();

  writestring( fdlancon, "LAN -- Hard Drive tests complete.\r\n"); 
  
//  ReadAndDisplay(0);
}


// this goes through an encapsulated semaphore
BYTE ReadSectorHD (unsigned int lba, unsigned char * buffer)
{
  return Read_Write_SectorHD (lba, buffer, READ);
}



// this goes through an encapsulated semaphore
BYTE WriteSectorHD (unsigned int lba, unsigned char * buffer)
{
  return Read_Write_SectorHD (lba, buffer, WRITE);
}



BYTE Read_Write_SectorHD (unsigned int lba, unsigned char * buffer, unsigned char rw)
{
  BYTE result;

  result = OSSemPend( &HDSem, 100/MS_PER_TICK );    // max wait 100ms
  if (result == OS_TIMEOUT)
  {
    printf("LAN -- Hard Drive timed out. lba = %08x rw = %d\n\r", lba, rw);
    return result;
  }

  if (rw == READ)
    _ReadSectorHD (lba, buffer); 
  else
    _WriteSectorHD (lba, buffer);
    
  OSSemPost( &HDSem );                              // set the semaphore back to 1
  
  return result; 
}



void ReadAndDisplay (unsigned int lba)
{
  unsigned int x, y;

  ReadSectorHD(lba, HDbuf);

    sprintf(buf, "\n\rLAN -- LBA = %x\n\r", lba);
    printf(buf);

  for (y = 0; y < 16; y++)
  {
    for (x = 0; x < 32; x+=2)
    {
      if (x != 30)
        sprintf(buf, "%02x%02x ", HDbuf[y*32+x], HDbuf[y*32+x+1]);
      else
      {
        sprintf(buf, "%02x%02x\n\r", HDbuf[y*32+x], HDbuf[y*32+x+1]);
      }
       printf(buf);
    }
  }
}


void ReadAndCheck (unsigned int lba)
{
  unsigned int x;

  ReadSectorHD(lba, HDbuf);

  for (x = 0; x < 512; x++) 
  {
    if (HDbuf[x] != x/2)
    {
      printf("LAN -- Bad Sector %u\r\n", lba);
    }
  }
}



void _ReadSectorHD (unsigned int lba, unsigned char * buffer)
{
  unsigned int        x;
  union short2bytes   s2b;

  WaitForHD();

  *spHD_SECTORCNT = 1;
  *spHD_LBALOW    = lba & 0xff;
  *spHD_LBAMID    = (lba & 0xff00) >> 8;
  *spHD_LBAHIGH   = (lba & 0xff0000) >> 16;
  *spHD_DEVREG    = 0x40 + ((lba >> 24) & 0xf);       // lba top
  WaitForHD();
  *spHD_COMMAND   = 0x20;                             // read sector
  
  for (x = 0; x < SECTOR_LEN; x+=2) 
  {
    WaitForHD();

    s2b.value   = *spHD_DATA;
    buffer[x]   = s2b.byte[0];
    buffer[x+1] = s2b.byte[1];
  }
}



void _WriteSectorHD (unsigned int lba, unsigned char * buffer)
{
  unsigned int        x;
  union short2bytes   s2b;

  WaitForHD();

  *spHD_SECTORCNT = 1;
  *spHD_LBALOW    = lba & 0xff;
  *spHD_LBAMID    = (lba & 0xff00) >> 8;
  *spHD_LBAHIGH   = (lba & 0xff0000) >> 16;
  *spHD_DEVREG    = 0x40 + ((lba >> 24) & 0xf);       // lba top
  WaitForHD();
  *spHD_COMMAND   = 0x30;                             // write sector
  
  for (x = 0; x < SECTOR_LEN; x+=2) 
  {
    s2b.byte[0] = buffer[x];
    s2b.byte[1] = buffer[x+1];

    WaitForHD();

    *spHD_DATA = s2b.value;
  }

  WaitForHD();
  *spHD_COMMAND   = 0xe7;                             // flush write cache
}



BYTE WriteBufferHD (unsigned int location, unsigned char * buffer, unsigned int length)
{
  unsigned int  StartSector, CurSector, StartByte, byte, SectorByte;
  BYTE          result;

  StartSector = (location >> 9);
  StartByte = location - (StartSector << 9);
  CurSector = StartSector;

  result = ReadSectorHD(CurSector, HDbuf);       // in case we start in the middle

  if (result == OS_TIMEOUT)
    return result;
    
  SectorByte = StartByte;

  for (byte = 0; byte < length; byte++)
  {
    HDbuf[SectorByte] = buffer[byte];

    SectorByte++;

    if (SectorByte >= SECTOR_LEN)
    {
      result = WriteSectorHD(CurSector, HDbuf);  // we are at the end of the sector
      if (result == OS_TIMEOUT)
        return result;
      CurSector++;                               // go to next one 
      SectorByte = 0;                            // back to the start byte for the new sector
      result = ReadSectorHD(CurSector, HDbuf);   // read the new sector into the buffer
      if (result == OS_TIMEOUT)
        return result;
    }
  }
  result = WriteSectorHD(CurSector, HDbuf);      // write the final sector
  if (result == OS_TIMEOUT)
    return result;
  return result;
}


BYTE ReadBufferHD (unsigned int location, unsigned char * buffer, unsigned int length)
{
  unsigned int  StartSector, CurSector, StartByte, byte, SectorByte;
  BYTE          result;
  
  StartSector = (location >> 9);
  StartByte = location - (StartSector << 9);
  CurSector = StartSector;

  result = ReadSectorHD(CurSector, HDbuf);       // read the first sector       
  if (result == OS_TIMEOUT)
    return result;

  SectorByte = StartByte;

  for (byte = 0; byte < length; byte++)
  {
    buffer[byte] = HDbuf[SectorByte];

    SectorByte++;

    if (SectorByte >= SECTOR_LEN)                // we are at the end of the sector
    {
      CurSector++;                               // go to next one
      SectorByte = 0;                            // back to the start byte for the new sector
      result = ReadSectorHD(CurSector, HDbuf);   // read the new sector into the buffer
      if (result == OS_TIMEOUT)
        return result;
    }
  }
  return result;
}

 
 
void WaitForHD (void)
{
  status = (*spHD_STATUS & (HDBUSY | HDRDY));
  if (status == HDRDY)
    return;

  do                                  // wait till hard drive is not busy
  {
    OSTimeDly(1);                     // wait 50 ms
    
    status = (*spHD_STATUS & (HDBUSY | HDRDY));
  } while (status != HDRDY);
}



void WaitForHDnBUSY (void)
{
  status = (*spHD_STATUS & (HDBUSY));
  if (status != HDBUSY)
    return;

  do                                  // wait till hard drive is not busy
  {
    OSTimeDly(1);                     // wait 50 ms
    
    status = (*spHD_STATUS & (HDBUSY));
  } while (status == HDBUSY);
}



void HDTest ( void )
{
  WaitForHD();

  sprintf(buf, "LAN -- Hard Drive is ready; status = %0x, mirror = %0x\r\n",
               *spHD_STATUS & 0xff, *spHD_ALTSTATUS & 0xff);
  writestring(fdlancon, buf);
  

  *spHD_LBALOW  = 0xa0;           // test only
  *spHD_LBAMID  = 0xb1;           // test only
  *spHD_LBAHIGH = 0xc2;           // test only
  *spHD_DEVREG  = 0xe0;           // make sure we don't screw up this register

  sprintf(buf, "LAN -- LBA low = %02x, mid = %02x, high = %02x, top = %02x\r\n",
               *spHD_LBALOW & 0xff, *spHD_LBAMID & 0xff, *spHD_LBAHIGH & 0xff, *spHD_DEVREG & 0xff);
  writestring(fdlancon, buf);

  WaitForHD();
 /* 
  *spHD_COMMAND = 0xec;           // get device characteristics

  printf("\r\n");
  for (int x = 0; x < 256; x++) 
  {
    WaitForHD();

    printf("%03d=%04x,", x, *spHD_DATA);
    if ((x & 7) == 7)
      printf("\r\n");
  }
*/

  OSSemPost( &HDSem );            // set the semaphore to 1

  WaitForHD();

  printf("LAN -- Hard Drive tests complete.\n\r"); 
/*
  unsigned int x;   
  for (x = 0; x < 512; x++) 
  {
    HDbuf[x] = x/2;
  }
  for (x = hdGAMES; x < hdNEXT; x++) 
  {
    WriteSectorHD (x, HDbuf);
    ReadAndCheck (x);
    if ((x & 0xf) == 0)
      printf(".");
  }*/
  
/*  
  printf("hdROLL = %ld\r\n", hdROLL);
  
  for (int x = 0; x < MAXINDEX; x++)
  {
    WriteRoll(x, x + 0x40000000);
    if ((x & 0xf) == 0)
      printf(".");
  }
  for (int x = 0; x < MAXINDEX; x++)
  {
    printf ("%08lx ", ReadRoll(x));
  }
*/
}






