/**********************************************************************
   Mark Butcher    Bsc (Hons) MPhil MIEE

   M.J.Butcher Consulting
   Obere Bahnhofstrasse 13, 5507 Mellingen

   www.uTasker.com      Skype: M_J_Butcher 

   ---------------------------------------------------------------------
   File:        spi_flash_m5223x_atmel.h [FREESCALE Coldfire]
   Project:     Single Chip Embedded Internet 
   ---------------------------------------------------------------------
   Copyright (C) M.J.Butcher Consulting 2004..2008
   *********************************************************************
   Extensively modified by Ewan MacPherson 04 Nov 2009
*/ 
 
/************************************************************************
 *                                                                      *
 * This code is for the QSPI port                                       *
 *                                                                      *
 * Copyright (c) 2009 E. D. MacPherson                                  *
 *                                                                      *
 ************************************************************************/
 
#include "predef.h"
#include <stdio.h>
#include <startnet.h>
#include "harddrives.h"
#include "main.h"
#include "defines.h"
#include "QSPIdefines.h"
#include "QSPI.h"
 
static void fnConfigSPI(void);

extern int        fdwancon;
char              buff[512];
char              SPI_buffer[SPI_FLASH_PAGE_LENGTH];
int               PARAMS_buffer[SPI_FLASH_PAGE_LENGTH/4];
      


// QSPI FLASH hardware interface
//
void fnSPI_command(unsigned char ucCommand, unsigned long ulPageNumberOffset, volatile unsigned char *ucData, MAX_FILE_LENGTH DataLength)
{
#define ucChipSelectLine QSPI_CS0_LINE
#define iChipSelect      0

  static int SPI_FLASH_Danger[1] = {0};                            // signal that the FLASH status should be checked before using since there is a danger that it is still busy {5} - array for compatibility
  unsigned char ucTxCount = 3;

  if (SPI_FLASH_Danger[iChipSelect] != 0) {                        // check whether the chip is ready to work, if not wait
    volatile unsigned char ucStatus;
    SPI_FLASH_Danger[iChipSelect] = 0;                             // device will no longer be busy after continuing
    do {
      fnSPI_command(READ_STATUS_REGISTER, 0, &ucStatus, 1);        // read busy status register 
    } while (!(ucStatus & STATUS_READY));                          // until no longer busy
  }

  _CLEARBITS(QSPI, ucChipSelectLine);                              // assert SS low before starting {3}
  QAR = QSPI_TRANSMIT_RAM_0;                                       // set address to first transmit location
  QDR = ucCommand;                                                 // prepare command

  switch (ucCommand) {
  case WRITE_BUFFER_1:                                             // 0x84 write data to the buffer
    QDR = 0;
    QDR = (unsigned char)(ulPageNumberOffset >> 8);
    QDR = (unsigned char)ulPageNumberOffset;
    break;
    
  // the following group uses page number and not char location
  case MAIN_TO_BUFFER_1:                                           // 0x53 command chip to copy a page from main memory to the buffer (400us transfer time)
  case PROG_FROM_BUFFER_1:                                         // 0x88 command programming buffer to FLASH (no erase)
  case ERASE_PROG_FROM_BUFFER_1:                                   // 0x83 erase before programming
  case PAGE_ERASE:                                                 // 0x81
  case BLOCK_ERASE:                                                // 0x50
    QDR = (unsigned char)(ulPageNumberOffset >> 6);
    QDR = (unsigned char)(ulPageNumberOffset << 2);
    QDR = 0;                                                       // dummy byte
    SPI_FLASH_Danger[iChipSelect] = 1;                             // mark that the device will be busy for some time
    break;

  case CONTINUOUS_ARRAY_READ:                                      // 0xe8 this is a legacy command for compatibility between B and D-devices
    QDR = (unsigned char)(ulPageNumberOffset >> 16);
    QDR = (unsigned char)(ulPageNumberOffset >> 8);
    QDR = (unsigned char)(ulPageNumberOffset);
    QDR = 0xff;                                                    // 4 dummy bytes needed before the device returns data
    QDR = 0xff;
    QDR = 0xff;
    QDR = 0xff;
    ucTxCount = 7;
    SPI_FLASH_Danger[iChipSelect] = 1;                             // in this case this marks that no 0xff have been set to tx when reading (it will be cleared before exiting)
    break;

  case READ_MANUFACTURER_ID:                                       // this only works on D-device
    QDR = 0xff;                                                    // dummy writes
    QDR = 0xff;
    QDR = 0xff;
    ucTxCount = 4;                                                 // fall through intentionally
  case READ_STATUS_REGISTER:                                       // read single byte from status register
    QDR = 0xff;
    if (READ_STATUS_REGISTER == ucCommand) {
      ucTxCount = 1;
    }
    QWR = ((0<<QSPI_START_SHIFT) | (ucTxCount<<QSPI_END_SHIFT));   // starting at queue 0 send/receive the required amount of entries
    QDLYR = QSPI_SPE;                                              // start transfer - bytes will be sent and read in
    while (!(QIR & QSPI_SPIF)) {};                                 // wait for transfer to complete
    QIR = QSPI_SPIF;                                               // clear interrupt flag
    QAR = QSPI_RECEIVE_RAM_0;                                      // set address to first receive location
    ucTxCount = (unsigned char)QDR;                                // clear command rx byte
    while (DataLength--) {
      *ucData++ = (unsigned char)QDR;
    }
    _SETBITS(QSPI, ucChipSelectLine);                              // deassert SS when complete
    return;

    // Note - no default here. This must never be called with non-supported command since it will hang.
  }

  QWR = ((0<<QSPI_START_SHIFT) | (ucTxCount<<QSPI_END_SHIFT));     // starting at queue 0 send/receive entries
  QDLYR = QSPI_SPE;                                                // start transfer - bytes will be sent and read in

  while (DataLength != 0) {                                        // while data bytes to be read or written
    int iDataCnt = 0;                                              // QSPI block counter

    if (ucTxCount != 0) {
      while (!(QIR & QSPI_SPIF)) {};                               // wait for transfer to complete
      QIR = QSPI_SPIF;                                             // clear interrupt flag
    }
  //QAR = QSPI_COMMAND_RAM_0;                                      // set address to first control location {2}
  //QDR = (QSPI_BITSE | QSPI_CS_3 | QSPI_CS_2 | QSPI_CS_1 | QSPI_CS_0); // no chip select since we control it via port - 8 bit access {2}
    QAR = QSPI_TRANSMIT_RAM_0;                                     // set address to first transmit location
    if (ucCommand != CONTINUOUS_ARRAY_READ) {                      // write type
      QDR = *ucData++;                                             // prepare data
      while (--DataLength != 0) {                                  // for each byte in the QSPI transfer block
        QDR = *ucData++;                                           // prepare data
        if (++iDataCnt >= 15) {                                    // maximum QSPI transfer length reached
          DataLength--;
          break;
        }
      }
      QWR = ((0<<QSPI_START_SHIFT) | (iDataCnt<<QSPI_END_SHIFT));  // starting at queue 0 send/receive programmed entries
      QDLYR = QSPI_SPE;                                            // start transfer - programmed number of bytes will be sent and read in
    }
    else {                                                         // read rather than write
      ucTxCount = 0;                                               // stop polling of status at begin of loop
      if (DataLength > 15) {                                       // maximum transfer length possible
        iDataCnt = 15;
        DataLength -= 16;                                          // complete block
      }
      else {                                                       // last transfer
        iDataCnt = (DataLength - 1);                               // last transfer block
        DataLength = 0;
      }
      if (SPI_FLASH_Danger[iChipSelect] != 0) {                    // this is performed only once in a read sequence (previously always true and so not optimal)
        int iSetIdle = iDataCnt;
        while (iSetIdle-- >= 0) {
          QDR = 0xff;                                              // fill output buffer with 0xff so that output remains '1'
        }
        SPI_FLASH_Danger[iChipSelect] = 0;                         // set value needed in case of CONTINUOUS_ARRAY_READ and stop more 0xff being written
      }
      QWR = ((0<<QSPI_START_SHIFT) | (iDataCnt<<QSPI_END_SHIFT));  // starting at queue 0 send/receive programmed entries
      QDLYR = QSPI_SPE;                                            // start transfer - programmed number of bytes will be sent and read in
      while (!(QIR & QSPI_SPIF)) {};                               // wait for transfer to complete
      QIR = QSPI_SPIF;                                             // clear interrupt flag

      QAR = QSPI_RECEIVE_RAM_0;                                    // set address to first receive location
      do {                                                         // collect each byte
        *ucData++ = (unsigned char)QDR;                            // read bytes to the return buffer
      } while (iDataCnt--);
    }
  }
  if (ucCommand != CONTINUOUS_ARRAY_READ) {                        // commands that have only written wait here for transmission to complete
    while (!(QIR & QSPI_SPIF)) {};                                 // wait for transfer to complete
    QIR = QSPI_SPIF;                                               // clear interrupt flag
  }

  _SETBITS(QSPI, ucChipSelectLine);                                // deassert SS when complete
}


// Check whether a known SPI FLASH device can be detected - called only once on start up
//
#define AT45DB041D                5                                // D-version of this device has been detected
#define AT45DB081D                6                                // D-version of this device has been detected
#define AT45DB161D                7                                // D-version of this device has been detected
#define AT45DB321D                8                                // D-version of this device has been detected
#define AT45DB641D                9                                // D-version of this device has been detected
#define NO_SPI_FLASH_AVAILABLE    0                                // No SPI FLASH detected

unsigned char fnCheckAT45dbxxx(void)
{
  volatile unsigned char ucID[4];
  int iRtn;

  OSTimeDly(1);                     // wait 50 ms
  iRtn = NO_SPI_FLASH_AVAILABLE;
   
  fnConfigSPI();

  fnSPI_command(READ_MANUFACTURER_ID, 0, ucID, sizeof(ucID));

  if (ucID[0] == MANUFACTURER_ID_ATMEL) {      // ATMEL D-part recognised
    switch (ucID[1]) {
    case DEVICE_ID_1_DATA_FLASH_4M:
      iRtn = AT45DB041D;
      break;
    case DEVICE_ID_1_DATA_FLASH_8M:
      iRtn = AT45DB081D;
      break;
    case DEVICE_ID_1_DATA_FLASH_16M:
      iRtn = AT45DB161D;
      break;
    case DEVICE_ID_1_DATA_FLASH_32M:
      iRtn = AT45DB321D;
      break;
    case DEVICE_ID_1_DATA_FLASH_64M:
      iRtn = AT45DB641D;
      break;
    default:                                 // possibly a larger part but we don't accept it
      break;
    }
  } // if (ucID[0] == MANUFACTURER_ID_ATMEL)
  
  return iRtn; 
}


static void fnConfigSPI(void)
{
  unsigned short Value;

  PAR_QSPI  = 0x1f;                         // clk, din, dout = QSPI. cs0, cs1 = GPIO
  PDDR_QSPI = 0x1d;                         // clk, dout, cs0, cs1 outputs, din input
  DSCR_QSPI = QSPI_MAX_DRIVE;
  _SETBITS(QSPI, QSPI_CS0_LINE);            // deassert cs0
   
                                            // set maximum possible SPI speed (20MHz with 80MHz clock)
  QMR = ((QSPI_MSTR | QSPI_8BITS | QSPI_CPOL | QSPI_CPHA) | MAX_SPI);
 
  Value = 16;                               // the depth of FIFO 
  QAR = QSPI_COMMAND_RAM_0;                 // set address to first control location {60}
  while (Value--) {                         // initialise all control registers
  	                                        // no chip selects since we control via port - 8 bit access
    QDR = (QSPI_BITSE | QSPI_CS_3 | QSPI_CS_2 | QSPI_CS_1 | QSPI_CS_0);
  }
  QWR = 0;                                  // clear wrap reg
  QIR = QSPI_SPIF;                          // clear interrupt flag
}


int fnWriteBytesFlash(unsigned char *ucDestination, unsigned char *ucData, MAX_FILE_LENGTH Length)
{
  // destination is assuming the Flash is a large array of chars
	
  // we have already checked that it is possible to modify the byte requested but we have to be sure that we only write 32 bits to long word addresses
  unsigned short usPageNumber;
  unsigned short usPageOffset;
  unsigned short usDataLength;
  unsigned short usPartLength;
  
  usPageNumber = fnGetSPI_FLASH_location(ucDestination, &usPageOffset);
    
  while (Length > 0) {
  	if (Length < MAX_CHARS) // 528 * 124
  	{
      usPartLength = (unsigned short)Length;
  	}
  	else
  	{
  	  usPartLength = MAX_CHARS;
  	}
  	usDataLength = usPartLength;
    while (usPartLength > 0) {
      if (usDataLength > usPartLength)
        usDataLength = usPartLength;
        
      if ((usPageOffset != 0) || (Length < SPI_FLASH_PAGE_LENGTH)) { 		// are we writing a partial page?
        fnSPI_command(MAIN_TO_BUFFER_1, usPageNumber, 0, 0); 				// copy main memory to buffer
      }
      if (usDataLength > (SPI_FLASH_PAGE_LENGTH - usPageOffset)) {
        usDataLength = (SPI_FLASH_PAGE_LENGTH - usPageOffset);
      }
      fnSPI_command(WRITE_BUFFER_1, usPageOffset, ucData, usDataLength);	// copy new content
      fnSPI_command(PROG_FROM_BUFFER_1, usPageNumber, 0, 0); 				// program to main memory
      usPartLength -= usDataLength;
      Length -= usDataLength;
      ucData += usDataLength;
      usPageNumber++;
      usPageOffset = 0;
    } // while
  }
  return 0;
}
 
 
unsigned short fnGetSPI_FLASH_location(unsigned char *ptrSector, unsigned short *usPageOffset)
{
  unsigned short usPageNumber;

  usPageNumber = (unsigned short)(((CAST_POINTER_ARITHMETIC)ptrSector)>>10); // the page the address is in
  if (usPageOffset != 0) {
    *usPageOffset = (unsigned short)((CAST_POINTER_ARITHMETIC)ptrSector & 0x3ff); // offset in the page
  }
  return usPageNumber;
}


int SendSPIToStreamAsBinary( int fd, int locPage)
{
  int            SendLength;
  int            chars_length;
  
  fnSPI_command(CONTINUOUS_ARRAY_READ, locPage++ * 1024, (unsigned char *)PARAMS_buffer, (unsigned short)SPI_FLASH_PAGE_LENGTH);	
  chars_length = PARAMS_buffer[0];
  
  while (chars_length > 0)
  {
    if (chars_length > SPI_FLASH_PAGE_LENGTH)
    {
      SendLength = SPI_FLASH_PAGE_LENGTH;
    }
    else
    {
      SendLength = chars_length;
    }
    fnSPI_command(CONTINUOUS_ARRAY_READ, locPage * 1024, (unsigned char *)SPI_buffer, (unsigned short)SendLength);	
    
    if ( writeall( fd, ( const char * ) SPI_buffer, SendLength ) < 0 )
    {
      return STREAM_UP_FAIL;
    }
    chars_length -= SendLength;
    locPage++;
  }
  return STREAM_UP_OK;
}



int ReadBinarySPIFromStream( int fd, int page_location )
{
  int                 nRead = 0, currentRead = 0;
  int                 rv, params_location;
   
  params_location = page_location ;
  page_location++;                    // skip the params page
  
  do
  {
    rv = ReadWithTimeout( fd, ( char * ) ( SPI_buffer ), SPI_FLASH_PAGE_LENGTH, 5 * TICKS_PER_SECOND );
    if ( rv > 0 )
    {
      nRead += rv;
      currentRead += rv;
    }
    if ( currentRead == SPI_FLASH_PAGE_LENGTH )
    {
      fnWriteBytesFlash((unsigned char *)(page_location++ * 1024), (unsigned char *)SPI_buffer, SPI_FLASH_PAGE_LENGTH);
      currentRead = 0;
    }
    else
    {
      fnWriteBytesFlash((unsigned char *)(page_location * 1024), (unsigned char *)SPI_buffer, currentRead);
      currentRead = 0;
    }
  }
  while ( rv > 0 );
   
  if ( nRead > 0 )
  {
    PARAMS_buffer[0] = nRead;	       // save length
    PARAMS_buffer[1] = 0;              // crc
    PARAMS_buffer[2] = 0x4d414d41;     // valid
    fnWriteBytesFlash((unsigned char *)(params_location * 1024), (unsigned char *)PARAMS_buffer, SPI_FLASH_PAGE_LENGTH);
    return STREAM_UP_OK;
  }
  return STREAM_UP_FAIL;
}


void fnSPI_Block_Erase(int start_block, int num_blocks)
{
  for (int x = 0; x < num_blocks; x++)
  {
    fnSPI_command(BLOCK_ERASE, start_block++ * PAGES_PER_BLOCK, 0,0);
  }
}


