
/************************************************************************
 *                                                                      *
 * This code is for the QSPI port                                       *
 *                                                                      *
 * Copyright (c) 2009 E. D. MacPherson                                  *
 *                                                                      *
 ************************************************************************/
 
#ifndef QSPIDEFINES_H_
#define QSPIDEFINES_H_

#define BUS_CLOCK (PLL_OUTPUT_FREQ_INT/POST_DIV)                 
#define PLL_OUTPUT_FREQ_INT    25000000                      // {7} run at crystal frequency with no PLL
#define OSCCLK                 25000000                      // 25 MHz oscillator / crystal input
#define POST_DIV               1
   
/***************************** PORTS ************************************/

#define IPSBAR 0x40000000
#define QSPI_ADD            (IPSBAR + 0x340)                             // QSPI module
#define PORT_MODULE_ADD     (IPSBAR + 0x100000)                          // Port Module

#define PORT_QS_BIT0        0x01
#define PORT_QS_BIT1        0x02
#define PORT_QS_BIT2        0x04
#define PORT_QS_BIT3        0x08
#define PORT_QS_BIT4        0x10
#define PORT_QS_BIT5        0x20
#define PORT_QS_BIT6        0x40
#define PORT_QS_BIT7        0x80

#define PODR_QSPI     *(volatile unsigned char *) (PORT_MODULE_ADD + 0x0a) // these are for the MCF5271 only
#define PDDR_QSPI     *(volatile unsigned char *) (PORT_MODULE_ADD + 0x1a)
#define PPDSDR_QSPI   *(volatile unsigned char *) (PORT_MODULE_ADD + 0x2a)
#define PCLRR_QSPI    *(volatile unsigned char *) (PORT_MODULE_ADD + 0x3a)
#define PAR_QSPI      *(volatile unsigned char *) (PORT_MODULE_ADD + 0x4a)
#define DSCR_QSPI     *(volatile unsigned char *) (PORT_MODULE_ADD + 0x54)

// SPI speed defines
#define MAX_SPI           2

#define _SETBITS(port, x)   PPDSDR_##port = x
#define _CLEARBITS(port, x) PCLRR_##port = ~x                             // {20} convert to positive logic by inverting                       

/****************************** SPI *************************************/

#define QSPI_CS0_LINE       PORT_QS_BIT3                                 // CS0 line used when SPI FLASH is enabled
#define _EXTENDED_CS
// QSPI module
//
#define QMR                 *(unsigned short *)(QSPI_ADD + 0x0)          // QSPI Mode register 
  #define QSPI_MSTR         0x8000                                       // Master operating mode (must be set)
  #define QSPI_DOHIE        0x4000                                       // Data Output Hi-Z between transfers
  #define QSPI_8BITS        0x2000                                       // 8 Bit transfer mode
  #define QSPI_10BITS       0x2800                                       // 10 bit transfer mode
  #define QSPI_11BITS       0x2c00                                       // 11 bit transfer mode
  #define QSPI_13BITS       0x3400                                       // 13 bit transfer mode
  #define QSPI_16BITS       0x0000                                       // 16 Bit transfer mode
  #define QSPI_CPOL         0x0200                                       // Clock polarity - inactive '1'
  #define QSPI_CPHA         0x0100                                       // Clock phase
  #define QSPI_BAUD_MASK    0x00ff                                       // Baud rate mask

#define QDLYR               *(unsigned short *)(QSPI_ADD + 0x4)          // QSPI Delay register
  #define QSPI_SPE          0x8000                                       // QSPI enable
#define QWR                 *(volatile unsigned short *)(QSPI_ADD + 0x8) // QSPI Wrap register 
  #define QSPI_HALT         0x8000                                       // halt transfers
  #define QSPI_WREN         0x4000                                       // wrap enable
  #define QSPI_WRTO         0x2000                                       // wrap to
  #define QSPI_CSIV         0x1000                                       // CS inactive level '1'
  #define QSPI_END_SHIFT    8
  #define QSPI_START_SHIFT  0
#define QIR                 *(volatile unsigned short *)(QSPI_ADD + 0xc) // QSPI Interrupt register 
  #define QSPI_SPIF         0x0001                                       // Finished flag
  #define QSPI_SPIFE        0x0100                                       // Interrupt enable
#define QAR                 *(volatile unsigned short *)(QSPI_ADD + 0x10)// QSPI Address register
  #define QSPI_TRANSMIT_RAM_0  0x00
  #define QSPI_TRANSMIT_RAM_1  0x01
  #define QSPI_TRANSMIT_RAM_2  0x02
  #define QSPI_TRANSMIT_RAM_3  0x03
  #define QSPI_TRANSMIT_RAM_4  0x04
  #define QSPI_TRANSMIT_RAM_5  0x05
  #define QSPI_TRANSMIT_RAM_6  0x06
  #define QSPI_TRANSMIT_RAM_7  0x07
  #define QSPI_TRANSMIT_RAM_8  0x08
  #define QSPI_TRANSMIT_RAM_9  0x09
  #define QSPI_TRANSMIT_RAM_10 0x0a
  #define QSPI_TRANSMIT_RAM_11 0x0b
  #define QSPI_TRANSMIT_RAM_12 0x0c
  #define QSPI_TRANSMIT_RAM_13 0x0d
  #define QSPI_TRANSMIT_RAM_14 0x0e
  #define QSPI_TRANSMIT_RAM_15 0x0f
  
  #define QSPI_RECEIVE_RAM_0   0x10
  #define QSPI_RECEIVE_RAM_1   0x11
  #define QSPI_RECEIVE_RAM_2   0x12
  #define QSPI_RECEIVE_RAM_3   0x13
  #define QSPI_RECEIVE_RAM_4   0x14
  #define QSPI_RECEIVE_RAM_5   0x15
  #define QSPI_RECEIVE_RAM_6   0x16
  #define QSPI_RECEIVE_RAM_7   0x17
  #define QSPI_RECEIVE_RAM_8   0x18
  #define QSPI_RECEIVE_RAM_9   0x19
  #define QSPI_RECEIVE_RAM_10  0x1a
  #define QSPI_RECEIVE_RAM_11  0x1b
  #define QSPI_RECEIVE_RAM_12  0x1c
  #define QSPI_RECEIVE_RAM_13  0x1d
  #define QSPI_RECEIVE_RAM_14  0x1e
  #define QSPI_RECEIVE_RAM_15  0x1f

  #define QSPI_COMMAND_RAM_0   0x20
  #define QSPI_COMMAND_RAM_1   0x21
  #define QSPI_COMMAND_RAM_2   0x22
  #define QSPI_COMMAND_RAM_3   0x23
  #define QSPI_COMMAND_RAM_4   0x24
  #define QSPI_COMMAND_RAM_5   0x25
  #define QSPI_COMMAND_RAM_6   0x26
  #define QSPI_COMMAND_RAM_7   0x27
  #define QSPI_COMMAND_RAM_8   0x28
  #define QSPI_COMMAND_RAM_9   0x29
  #define QSPI_COMMAND_RAM_10  0x2a
  #define QSPI_COMMAND_RAM_11  0x2b
  #define QSPI_COMMAND_RAM_12  0x2c
  #define QSPI_COMMAND_RAM_13  0x2d
  #define QSPI_COMMAND_RAM_14  0x2e
  #define QSPI_COMMAND_RAM_15  0x2f
  
#define QDR                *(volatile unsigned short *)(QSPI_ADD + 0x14) // QSPI Data register 

//Command RAM register defines
#define QSPI_CONT           0x8000                                       // CS continuous
#define QSPI_BITSE          0x4000                                       // Number of bits defined in QMR (otherwise 8)
#define QSPI_QUEUE_8BIT     0x0000                                       // 8 bit transfer
#define QSPI_DT             0x2000                                       // Delay after transfer 
#define QSPI_DSCK           0x1000                                       // Chip select delay enable
#define QSPI_CS_3           0x0800                                       // CS 3 state '1' during transfer
#define QSPI_CS_2           0x0400                                       // CS 2 state '1' during transfer
#define QSPI_CS_1           0x0200                                       // CS 1 state '1' during transfer
#define QSPI_CS_0           0x0100                                       // CS 0 state '1' during transfer

#define QSPI_MAX_DRIVE           1

    // This is the main interface code to the ATMEL SPI FLASH device

/* =================================================================== */
/*                           AT45DB041 driver                          */
/* =================================================================== */

#define MANUFACTURER_ID_ATMEL      0x1F                                  // D-version manufacturer's ID
#define DEVICE_ID_1_DATA_FLASH_4M  0x24                                  // D-version FLASH size - 4MBit / 0.5MegByte
#define DEVICE_ID_1_DATA_FLASH_8M  0x25                                  // D-version FLASH size - 8MBit / 1MegByte
#define DEVICE_ID_1_DATA_FLASH_16M 0x26                                  // D-version FLASH size - 16MBit / 2MegByte
#define DEVICE_ID_1_DATA_FLASH_32M 0x27                                  // C/D-version FLASH size - 32MBit / 4MegByte
#define DEVICE_ID_1_DATA_FLASH_64M 0x28                                  // D-version FLASH size - 64MBit / 8MegByte


#define READ_STATUS_REGISTER     0xd7
  #define STATUS_READY           0x80
  #define STATUS_MISMATCH        0x40
  #define STATUS_1MEG            0x0c                                    // expected in 1Meg bit device
  #define STATUS_2MEG            0x14                                    // expected in 2Meg bit device
  #define STATUS_4MEG            0x1c                                    // expected in 4Meg bit device
  #define STATUS_8MEG            0x24                                    // expected in 8Meg bit device
  #define STATUS_16MEG           0x2c                                    // expected in 16Meg bit device
  #define STATUS_32MEG           0x34                                    // expected in 32Meg bit device
  #define STATUS_64MEG           0x3c                                    // expected in 64Meg bit device
  #define STATUS_PROTECTED_SECT  0x02                                    // sector is protected (D-device)
  #define STATUS_PAGE_SIZE       0x01                                    // 256 byte pages size (rather than 264)
#define WRITE_BUFFER_1           0x84
#define WRITE_BUFFER_2           0x87
#define ERASE_PROG_FROM_BUFFER_1 0x83
#define ERASE_PROG_FROM_BUFFER_2 0x86
#define PROG_FROM_BUFFER_1       0x88
#define PROG_FROM_BUFFER_2       0x89
#define PAGE_ERASE               0x81
#define BLOCK_ERASE              0x50
#define PROG_THROUGH_BUFFER_1    0x82                                    // combines WRITE_BUFFER and ERASE_PROG_FROM_BUFFER
#define PROG_THROUGH_BUFFER_2    0x85
#define MAIN_TO_BUFFER_1         0x53                                    // copy page of main memory to buffer
#define MAIN_TO_BUFFER_2         0x55
#define COMPARE_WITH_BUFFER_1    0x60                                    // compare page of main memory with buffer
#define COMPARE_WITH_BUFFER_2    0x61
#define AUTO_PAGE_REWRITE_1      0x58                                    // combines MAIN_TO_BUFFER and ERASE_PROG_FROM_BUFFER and can be used to refresh pages (it is recommeded to refresh each page in a sector at least every 10'000 page erase/program operations of the sector)
#define AUTO_PAGE_REWRITE_2      0x59
#define CONTINUOUS_ARRAY_READ    0xe8


// Command supported only by D-device
#define SECTOR_ERASE             0x7c
#define READ_ARRAY_LOW_FREQ      0x03
#define READ_MANUFACTURER_ID     0x9f
// Chip erase is not spported due to bugs in some chips - sector erase is used instead

#define SPI_FLASH_PAGE_LENGTH    528
#define CAST_POINTER_ARITHMETIC  unsigned long                           // M5223X uses 32 bit pointers

#define STREAM_UP_OK     (1)
#define STREAM_UP_FAIL   (0)
#define USER_FLASH_SIZE  (1000 * SPI_FLASH_PAGE_LENGTH)
#define MAX_CHARS        0xffc0
#define PAGES_PER_BLOCK  8

#endif /*QSPIDEFINES_H_*/
