/******************************************************************************
 *
 * Copyright 1998-2004 NetBurner, Inc.  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or it's derivitives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non NetBurner Hardware. Please contact Licensing@Netburner.com 
 *   for more information.
 *
 *   NetBurner makes no representation or warranties with respect to the 
 *   performance of this computer program, and specifically disclaims any 
 *   responsibility for any damages, special or consequential, connected 
 *   with the use of this program.
 *
 *   NetBurner, Inc
 *   5405 Morehouse Drive
 *   San Diego Ca, 92121
 *   http://www.netburner.com
 *
 *****************************************************************************/
// LAN
  
#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <serial.h>
#include <iosys.h>
#include <utils.h>
#include <ip.h>
#include <tcp.h>
#include <bsp.h>
#include <autoupdate.h>
#include <StreamUpdate.h>
#include <taskmon.h>
#include <..\MOD5270\system\sim5270.h>
#include "defines.h"
#include "QSPI.h"
#include "harddrives.h"
#include "network.h"
#include "main.h"



// semaphores
extern OS_SEM             HDSem;

extern "C"
{
  void UserMain( void *pd );
}

int                     fdlancon, fdstream, fdlanbu;
const char              *AppName = "LAN Server";
extern char             buf[];


static DWORD  NetworkStk[USER_TASK_STK_SIZE] __attribute__( ( aligned( 4 ) ) );
 
 
void UserMain( void *pd )
{
//                        IP Address             IP Mask             Gateway
//                     192.168.254.3       255.255.255.0     192.168.254.254
  InitializeStack((IPADDR)0xc0a8fe03, (IPADDR)0x00000000, (IPADDR)0xc0a8fefe);  // Initialize TCP/IP Stack

  OSChangePrio( MAIN_PRIO );  // Set UserMain task priority to the default (50)
  StartTcpUpdate(15);         // Enable tcpupdate capability on internet

  EnableTaskMonitor(); 
 
  OSSemaphores_Create();      // before any other tasks start

  InitSerial();
  Init_CS();
  InitAndTest();
  
  OSTasks_Create();
   
  while (1)
  {    	
    if (fdstream > 0)
    {
      CheckStream();
    }
    if (fdlancon > 0)
    {
      CheckCon();
    }    
  }// while (1)
}//UserMain
  

void CheckCon (void)
{
  char               buffer[40];
  int                n;

  if (dataavail( fdlancon ))
  {
    n = read( fdlancon, buffer, 40 );
    write( fdlancon, buffer, n );
  }
}


void CheckStream (void)
{
  static int         started = FALSE;
  char               buffer[40];
  int                n, pageLocation;
  static int         m, fileLength;
  char               * pLoc, * pLen;
  
  if (dataavail( fdstream ) && (started == FALSE))
  {
    n = ReadWithTimeout( fdstream, buffer, 1, TICKS_PER_SECOND );     
    if ((buffer[0] == '/') && ( n > 0 ))
    {
      n = ReadWithTimeout( fdstream, buffer, 1, TICKS_PER_SECOND );     
      if ((buffer[0] == 'f') && ( n > 0 )) // here we do a file save to QSPI
      {
        started = TRUE;
        m = n = 0;
        pLoc = (char *)&pageLocation;
        while (n < 4)
        {
          m = ReadWithTimeout( fdstream, pLoc, 4, TICKS_PER_SECOND );
          n += m;
          pLoc += m;
        }
        m = n = 0;
        pLen = (char *)&fileLength;
        while (n < 4)
        {
          m = ReadWithTimeout( fdstream, pLen, 4, TICKS_PER_SECOND );
          n += m;
          pLen += m;
        }
        sprintf(buf, "LAN -- File Started, Start Page = %d, File Length = %d\r\n", pageLocation, fileLength);
        writestring( fdlancon, buf );
        m = n = 0;
      }
      else if ((buffer[0] == 'u') && ( n > 0 )) // here we do an update to the LAN software
      {
        writestring( fdlancon, "LAN -- Starting Update\r\n" );
        if (ReadS19ApplicationCodeFromStream( fdstream ) == STREAM_UP_OK)
        {
          writestring( fdlancon, "LAN -- Software Updated\r\n" );
          writestring( fdlancon, "LAN -- Rebooting\r\n" );
          OSTimeDly(TICKS_PER_SECOND * 5);
          ForceReboot();
        }
        else
        {
           writestring( fdlancon, "LAN -- Update Failed\r\n" );
        }
      }
    }
  }// if (dataavail( fdstream ) && (started == FALSE))
  else if (dataavail( fdstream ) && (started == TRUE))
  {
    // save file to QSPI here
    n = ReadWithTimeout( fdstream, buffer, 40, TICKS_PER_SECOND );
    m += n;
    if (m >= fileLength)
    {
      started = FALSE;
      sprintf(buf, "LAN -- File Ended, File Length = %d\r\n", m);
      writestring( fdlancon, buf );
    }
  }
}



void InitAndTest (void)
{
	char buf[128];
	
	if(fnCheckAT45dbxxx())
	{
	  sprintf(buf, "LAN -- AT45DB321 OK\r\n");
	  writestring(fdlancon, buf);
	}
	 
    HDInit();
}

void Init_CS (void)
{
   sim.cs[2].csar = ( CS2 >> 16 );            // HD 
   sim.cs[2].cscr = 0x3d80;                   /* 0011 1101 1000  0000 */
   sim.cs[2].csmr = 0x00010001;
}

 
void OSSemaphores_Create (void)
{
  OSSemInit( &HDSem, 0 );           // create but do not permit to be used yet ( = 0 )
}

void InitSerial (void)
{
#define BAUD              (57600)
#define STOP_BITS         (1)
#define DATA_BITS         (8)
#define ENABLED           (1)
#define DISABLED          (0)
  
  //Close the serial ports in case they are already open.
  SerialClose( 0 );
  SerialClose( 1 );
  SerialClose( 2 );

  //Open the serial ports
  fdstream = OpenSerial( 0, BAUD, STOP_BITS, DATA_BITS, eParityNone ); // stream
  fdlancon = OpenSerial( 1, BAUD, STOP_BITS, DATA_BITS, eParityNone ); // LAN CON
  fdlanbu  = OpenSerial( 2, BAUD, STOP_BITS, DATA_BITS, eParityNone ); // BACKUP LAN 
   
  //set SerialEnableHwRxFlow & SerialEnableHwRxFlow here
  SerialEnableHwRxFlow( 0, ENABLED ); 
  SerialEnableHwTxFlow( 0, ENABLED ); 
  // do serial stuff here
  if ( (fdlancon > 0) && (fdlanbu > 0) && (fdstream > 0) )
    writestring(fdlancon, "\r\nLAN -- Serial ports OK\r\n" ); 
}

void OSTasks_Create (void)
{
  OSTaskCreate( Network,
                0,
                ( void * ) &NetworkStk[USER_TASK_STK_SIZE],
                ( void * ) NetworkStk,
                NETWORK_PRIO );
}



